@@ -156,6 +156,15 @@ class ContainerBuilder:
             compress=False,
             shasum=False
         )
+        self.result.add(
+            key='image_changes',
+            filename=self.system_setup.export_package_changes(
+                self.target_dir
+            ),
+            use_for_bundle=True,
+            compress=True,
+            shasum=False
+        )
         self.result.add(
             key='image_verified',
             filename=self.system_setup.export_package_verification(
