@@ -121,7 +121,7 @@ class DiskBuilder:
         if custom_args and 'signing_keys' in custom_args:
             self.signing_keys = custom_args['signing_keys']
 
-        self.boot_image = BootImage(
+        self.boot_image = BootImage.new(
             xml_state, target_dir, root_dir, signing_keys=self.signing_keys
         )
         self.firmware = FirmWare(
@@ -251,7 +251,7 @@ class DiskBuilder:
         )
 
         # create the bootloader instance
-        self.bootloader_config = BootLoaderConfig(
+        self.bootloader_config = BootLoaderConfig.new(
             self.bootloader, self.xml_state, root_dir=self.root_dir,
             boot_dir=self.root_dir, custom_args={
                 'targetbase':
@@ -514,6 +514,15 @@ class DiskBuilder:
             compress=False,
             shasum=False
         )
+        self.result.add(
+            key='image_changes',
+            filename=self.system_setup.export_package_changes(
+                self.target_dir
+            ),
+            use_for_bundle=True,
+            compress=True,
+            shasum=False
+        )
         self.result.add(
             key='image_verified',
             filename=self.system_setup.export_package_verification(
@@ -647,7 +656,7 @@ class DiskBuilder:
             exclude_list.append(
                 '{0}/.*'.format(self.spare_part_mountpoint.lstrip(os.sep))
             )
-        if 'boot' in device_map and self.bootloader == 'grub2_s390x_emu':
+        if 'boot' in device_map and 's390' in self.arch:
             exclude_list.append('boot/zipl/*')
             exclude_list.append('boot/zipl/.*')
         elif 'boot' in device_map:
@@ -702,9 +711,8 @@ class DiskBuilder:
             if not boot_filesystem:
                 boot_filesystem = self.requested_filesystem
             boot_directory = self.root_dir + '/boot/'
-            if self.bootloader == 'grub2_s390x_emu':
+            if 's390' in self.arch:
                 boot_directory = self.root_dir + '/boot/zipl/'
-                boot_filesystem = 'ext2'
             log.info(
                 'Creating boot(%s) filesystem on %s',
                 boot_filesystem, device_map['boot'].get_device()
@@ -895,7 +903,7 @@ class DiskBuilder:
             custom_root_mount_args, fs_check_interval
         )
         if device_map.get('boot'):
-            if self.bootloader == 'grub2_s390x_emu':
+            if 's390' in self.arch:
                 boot_mount_point = '/boot/zipl'
             else:
                 boot_mount_point = '/boot'
@@ -1096,7 +1104,7 @@ class DiskBuilder:
                 "custom arguments for bootloader installation %s",
                 custom_install_arguments
             )
-            bootloader = BootLoaderInstall(
+            bootloader = BootLoaderInstall.new(
                 self.bootloader, self.root_dir, self.disk.storage_provider,
                 custom_install_arguments
             )
