@@ -1,6 +1,7 @@
 import io
 import os
 import logging
+from collections import namedtuple
 from mock import (
     patch, call, MagicMock, Mock
 )
@@ -21,7 +22,8 @@ from kiwi.exceptions import (
     KiwiTemplateError,
     KiwiBootLoaderGrubDataError,
     KiwiBootLoaderGrubFontError,
-    KiwiBootLoaderGrubModulesError
+    KiwiBootLoaderGrubModulesError,
+    KiwiDiskGeometryError
 )
 
 
@@ -36,6 +38,9 @@ class TestBootLoaderConfigGrub2:
     def setup(
         self, mock_machine, mock_theme, mock_firmware
     ):
+        self.command_type = namedtuple(
+            'command_return_type', ['output']
+        )
         self.find_grub = {}
         self.os_exists = {
             'root_dir/boot/grub2/fonts/unicode.pf2': True,
@@ -104,7 +109,9 @@ class TestBootLoaderConfigGrub2:
         )
         self.bootloader = BootLoaderConfigGrub2(
             self.state, 'root_dir', None, {
-                'grub_directory_name': 'grub2', 'boot_is_crypto': True
+                'grub_directory_name': 'grub2',
+                'boot_is_crypto': True,
+                'targetbase': 'rootdev'
             }
         )
         self.bootloader.cmdline = 'some-cmdline root=UUID=foo'
@@ -187,6 +194,16 @@ class TestBootLoaderConfigGrub2:
         bootloader = BootLoaderConfigGrub2(xml_state, 'root_dir')
         assert bootloader.arch == mock_machine.return_value
 
+    @patch('platform.machine')
+    def test_post_init_s390_platform(self, mock_machine):
+        xml_state = MagicMock()
+        xml_state.build_type.get_firmware = Mock(
+            return_value=None
+        )
+        mock_machine.return_value = 's390x'
+        bootloader = BootLoaderConfigGrub2(xml_state, 'root_dir')
+        assert bootloader.arch == mock_machine.return_value
+
     @patch('platform.machine')
     def test_post_init_arm64_platform(self, mock_machine):
         xml_state = MagicMock()
@@ -239,6 +256,19 @@ class TestBootLoaderConfigGrub2:
         mock_setup_default_grub.assert_called_once_with()
         mock_setup_sysconfig_bootloader.assert_called_once_with()
 
+    @patch.object(BootLoaderConfigGrub2, '_setup_default_grub')
+    @patch.object(BootLoaderConfigGrub2, '_setup_sysconfig_bootloader')
+    @patch.object(BootLoaderConfigGrub2, '_setup_zipl2grub_conf')
+    def test_write_meta_data_s390(
+        self, mock_setup_zipl2grub_conf, mock_setup_sysconfig_bootloader,
+        mock_setup_default_grub
+    ):
+        self.bootloader.arch = 's390x'
+        self.bootloader.write_meta_data()
+        mock_setup_default_grub.assert_called_once_with()
+        mock_setup_sysconfig_bootloader.assert_called_once_with()
+        mock_setup_zipl2grub_conf.assert_called_once_with()
+
     @patch('os.path.exists')
     @patch.object(BootLoaderConfigGrub2, '_copy_grub_config_to_efi_path')
     @patch('kiwi.bootloader.config.grub2.Command.run')
@@ -314,6 +344,171 @@ class TestBootLoaderConfigGrub2:
             'config_file', 'root_dir/EFI/fedora/grub.cfg'
         )
 
+    @patch('shutil.copy')
+    @patch('os.path.exists')
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    def test_setup_zipl2grub_conf_512_byte_target(
+        self, mock_Command_run, mock_exists, mock_shutil_copy
+    ):
+        path_return_values = [True, False]
+
+        def path_exists(arg):
+            return path_return_values.pop(0)
+
+        command = Mock()
+        command.output = '  2048'
+        self.bootloader.target_table_type = 'msdos'
+        mock_Command_run.return_value = command
+        mock_exists.side_effect = path_exists
+        xml_state = MagicMock()
+        xml_state.get_build_type_bootloader_targettype = Mock(
+            return_value='FBA'
+        )
+        xml_state.build_type.get_target_blocksize = Mock(
+            return_value=None
+        )
+        self.bootloader.xml_state = xml_state
+        with open('../data/etc/default/zipl2grub.conf.in') as zipl_grub:
+            zipl_config = zipl_grub.read()
+        with patch('builtins.open', create=True) as mock_open:
+            file_handle = mock_open.return_value.__enter__.return_value
+            file_handle.read.return_value = zipl_config
+            self.bootloader._setup_zipl2grub_conf()
+            assert \
+                '    targettype = FBA\n' \
+                '    targetbase = rootdev\n' \
+                '    targetblocksize = 512\n' \
+                '    targetoffset = 2048' \
+                in file_handle.write.call_args[0][0]
+        mock_shutil_copy.assert_called_once_with(
+            'root_dir/etc/default/zipl2grub.conf.in',
+            'root_dir/etc/default/zipl2grub.conf.in.orig'
+        )
+        path_return_values = [True, True]
+        mock_shutil_copy.reset_mock()
+        with patch('builtins.open', create=True) as mock_open:
+            file_handle = mock_open.return_value.__enter__.return_value
+            file_handle.read.return_value = zipl_config
+            self.bootloader._setup_zipl2grub_conf()
+        mock_shutil_copy.assert_called_once_with(
+            'root_dir/etc/default/zipl2grub.conf.in.orig',
+            'root_dir/etc/default/zipl2grub.conf.in'
+        )
+
+    @patch('shutil.copy')
+    @patch('os.path.exists')
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    def test_setup_zipl2grub_conf_4096_byte_target(
+        self, mock_Command_run, mock_exists, mock_shutil_copy
+    ):
+        path_return_values = [True, False]
+        command_return_values = [
+            self.command_type(
+                output='  blocks per track .....: 12\n'
+            ),
+            self.command_type(
+                output=' /dev/loop01 2 6401 6400 1 Linux native\n'
+            ),
+            self.command_type(
+                output='  cylinders ............: 10017\n'
+            ),
+            self.command_type(
+                output='  tracks per cylinder ..: 15\n'
+            ),
+            self.command_type(
+                output='  blocks per track .....: 12\n'
+            )
+        ]
+
+        def path_exists(arg):
+            return path_return_values.pop(0)
+
+        def command_run(arg):
+            return command_return_values.pop(0)
+
+        self.bootloader.target_table_type = 'dasd'
+        mock_Command_run.side_effect = command_run
+        mock_exists.side_effect = path_exists
+        xml_state = MagicMock()
+        xml_state.get_build_type_bootloader_targettype = Mock(
+            return_value='CDL'
+        )
+        xml_state.build_type.get_target_blocksize = Mock(
+            return_value=4096
+        )
+        self.bootloader.xml_state = xml_state
+        with open('../data/etc/default/zipl2grub.conf.in') as zipl_grub:
+            zipl_config = zipl_grub.read()
+        with patch('builtins.open', create=True) as mock_open:
+            file_handle = mock_open.return_value.__enter__.return_value
+            file_handle.read.return_value = zipl_config
+            self.bootloader._setup_zipl2grub_conf()
+            assert \
+                '    targettype = CDL\n' \
+                '    targetbase = rootdev\n' \
+                '    targetblocksize = 4096\n' \
+                '    targetoffset = 24\n' \
+                '    targetgeometry = 10017,15,12' \
+                in file_handle.write.call_args[0][0]
+
+        assert mock_Command_run.call_args_list == [
+            call(
+                [
+                    'bash', '-c',
+                    'fdasd -f -p rootdev | grep "blocks per track"'
+                ]
+            ),
+            call(
+                [
+                    'bash', '-c',
+                    'fdasd -f -s -p rootdev | grep "^ " | '
+                    'head -n 1 | tr -s " "'
+                ]
+            ),
+            call(
+                [
+                    'bash', '-c', 'fdasd -f -p rootdev | grep "cylinders"'
+                ]
+            ),
+            call(
+                [
+                    'bash', '-c',
+                    'fdasd -f -p rootdev | grep "tracks per cylinder"'
+                ]
+            ),
+            call(
+                [
+                    'bash', '-c',
+                    'fdasd -f -p rootdev | grep "blocks per track"'
+                ]
+            )
+        ]
+
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    @patch.object(BootLoaderConfigGrub2, '_get_dasd_disk_geometry_element')
+    def test_get_partition_start_raises(
+        self, mock_get_dasd_disk_geometry_element, mock_Command_run
+    ):
+        self.bootloader.target_table_type = 'dasd'
+        mock_Command_run.return_value = self.command_type(
+            output='bogus data'
+        )
+        with raises(KiwiDiskGeometryError):
+            self.bootloader._get_partition_start('/dev/disk')
+
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    def test_get_dasd_disk_geometry_element_raises(
+        self, mock_Command_run
+    ):
+        self.bootloader.target_table_type = 'dasd'
+        mock_Command_run.return_value = self.command_type(
+            output='bogus data'
+        )
+        with raises(KiwiDiskGeometryError):
+            self.bootloader._get_dasd_disk_geometry_element(
+                '/dev/disk', 'tracks per cylinder'
+            )
+
     @patch('os.path.exists')
     @patch('kiwi.bootloader.config.grub2.SysConfig')
     @patch('kiwi.bootloader.config.grub2.Command.run')
@@ -467,7 +662,7 @@ class TestBootLoaderConfigGrub2:
             file_handle_grubenv = \
                 mock_open_grubenv.return_value.__enter__.return_value
 
-            file_handle_grub.read.return_value = 'root=rootdev'
+            file_handle_grub.read.return_value = 'root=PARTUUID=xx'
             file_handle_grubenv.read.return_value = 'root=rootdev'
             file_handle_menu.read.return_value = 'options foo bar'
 
@@ -501,7 +696,7 @@ class TestBootLoaderConfigGrub2:
                     '    fi\n'
                     'fi\n'
                 ),
-                call('root=rootdev'),
+                call('root=PARTUUID=xx'),
                 # second write of grub.cfg, setting overlay root
                 call('root=overlay:UUID=ID')
             ]
@@ -916,6 +1111,39 @@ class TestBootLoaderConfigGrub2:
             ]
         )
 
+    @patch('kiwi.bootloader.config.base.BootLoaderConfigBase.get_boot_path')
+    @patch('kiwi.bootloader.config.grub2.Command.run')
+    @patch('kiwi.bootloader.config.grub2.DataSync')
+    @patch('os.path.exists')
+    @patch('platform.machine')
+    def test_setup_disk_boot_images_s390(
+        self, mock_machine, mock_exists, mock_sync,
+        mock_command, mock_get_boot_path
+    ):
+        mock_get_boot_path.return_value = '/boot'
+        mock_machine.return_value = 's390x'
+        self.bootloader.arch = 's390x'
+        self.firmware.efi_mode = Mock(
+            return_value=None
+        )
+        self.bootloader.xen_guest = False
+        self.os_exists['root_dir/boot/grub2/fonts/unicode.pf2'] = False
+        self.os_exists['root_dir/usr/share/grub2/unicode.pf2'] = True
+
+        def side_effect(arg):
+            return self.os_exists[arg]
+
+        mock_exists.side_effect = side_effect
+
+        self.bootloader.setup_disk_boot_images('0815')
+
+        mock_command.assert_called_once_with(
+            [
+                'cp', 'root_dir/usr/share/grub2/unicode.pf2',
+                'root_dir/boot/grub2/fonts'
+            ]
+        )
+
     @patch('kiwi.bootloader.config.base.BootLoaderConfigBase.get_boot_path')
     @patch('kiwi.bootloader.config.grub2.Command.run')
     @patch('os.path.exists')
