@@ -134,14 +134,14 @@ class TestDiskBuilder:
             return_value=True
         )
         self.bootloader_install = mock.Mock()
-        kiwi.builder.disk.BootLoaderInstall = mock.MagicMock(
+        kiwi.builder.disk.BootLoaderInstall.new = mock.MagicMock(
             return_value=self.bootloader_install
         )
         self.bootloader_config = mock.Mock()
         self.bootloader_config.get_boot_cmdline = mock.Mock(
             return_value='boot_cmdline'
         )
-        kiwi.builder.disk.BootLoaderConfig = mock.MagicMock(
+        kiwi.builder.disk.BootLoaderConfig.new = mock.MagicMock(
             return_value=self.bootloader_config
         )
         kiwi.builder.disk.DiskSetup = mock.MagicMock(
@@ -156,7 +156,7 @@ class TestDiskBuilder:
             kernel_name='linux.vmx',
             initrd_name='initrd.vmx'
         )
-        kiwi.builder.disk.BootImage = mock.Mock(
+        kiwi.builder.disk.BootImage.new = mock.Mock(
             return_value=self.boot_image_task
         )
         self.firmware = mock.Mock()
@@ -644,10 +644,6 @@ class TestDiskBuilder:
 
         self.bootloader_config.write.assert_called_once_with()
 
-        assert mock_fs.call_args_list[1] == call(
-            'ext2', self.device_map['boot'], 'root_dir/boot/zipl/'
-        )
-
     @patch('kiwi.builder.disk.FileSystem.new')
     @patch('kiwi.builder.disk.Command.run')
     @patch('kiwi.builder.disk.Defaults.get_grub_boot_directory_name')
